import uuid

from Crypto import Cipher
from Crypto.Cipher import AES
from Crypto import Random as CryptoRandom

from ..base import BaseTest
from ...lib.Encryption import Encryption


class Test(BaseTest):

    def setUp(self):
        # Generate a fake key
        self.key = str(uuid.uuid4()).encode()

        self.enc2 = Encryption(self.key)

    def test_digest_key(self):
        dk = self.enc2.digest_key()
        self.assertIsInstance(dk, bytes)
        self.assertEqual(len(dk), 32)

    def test_digest_key_2(self):
        salt = self.enc2.gen_salt()
        self.enc2.set_salt(salt)
        dk = self.enc2.digest_key()
        self.assertIsInstance(dk, bytes)
        self.assertEqual(len(dk), 32)

    def test_get_aes(self):
        IV = CryptoRandom.new().read(AES.block_size)
        self.assertIsInstance(self.enc2.get_aes(IV), Cipher._mode_cbc.CbcMode)

    def test_gen_salt(self):
        salt = self.enc2.gen_salt()
        self.assertIsInstance(salt, bytes)
        self.assertTrue(len(salt) > 0)
        self.assertEqual(self.enc2.salted_key, salt + self.key)

    def test_gen_salt_2(self):
        salt = self.enc2.gen_salt(False)
        self.assertIsInstance(salt, bytes)
        self.assertTrue(len(salt) > 0)

    def test_set_salt(self):
        salt = self.enc2.gen_salt()
        self.assertEqual(self.enc2.salted_key, salt + self.key)

    def test_encrypt(self):
        secret_string = b'my secret string'
        encrypted = self.enc2.encrypt(secret_string)
        self.assertIsInstance(encrypted, bytes)
        self.assertTrue(len(encrypted) > 0)

    def test_encrypt_2(self):
        secret_string = b'my secret string'
        salt = self.enc2.gen_salt()
        encrypted = self.enc2.encrypt(secret_string)
        self.assertIsInstance(encrypted, bytes)
        self.assertTrue(len(encrypted) > 0)

    def test_decrypt(self):
        secret_string = b'my secret string'
        encrypted = self.enc2.encrypt(secret_string)
        self.assertEqual(self.enc2.decrypt(encrypted), secret_string)

    def test_decrypt_2(self):
        secret_string = b'my secret string'
        salt = self.enc2.gen_salt()
        encrypted = self.enc2.encrypt(secret_string)
        self.enc2.set_salt(salt)
        self.assertEqual(self.enc2.decrypt(encrypted), secret_string)

    def test_decrypt_3(self):
        # Test encrypting with a salt but decrypting without the salt
        secret_string = b'my secret string'
        salt = self.enc2.gen_salt()
        self.enc2.set_salt(salt)
        encrypted = self.enc2.encrypt(secret_string)
        self.assertRaises(ValueError, self.enc2.decrypt, encrypted)



import base64
import string
from random import randint, choice

from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto import Random as CryptoRandom


class Encryption():

    def __init__(self, key):
        self.key = key  # Key in bytes
        self.salted_key = None  # Placeholder for optional salted key

    def digest_key(self):
        """
            Use SHA-256 over our key to get a proper-sized AES key
        """

        # Add optional salt to key
        key = self.key
        if self.salted_key:
            key = self.salted_key

        return SHA256.new(key).digest()

    def get_aes(self, IV):
        """
            AES instance
        """

        return AES.new(self.digest_key(), AES.MODE_CBC, IV)

    def gen_salt(self, set_=True):
        """
            Generate a random salt
        """

        min_char = 8
        max_char = 12
        allchar = string.ascii_letters + string.punctuation + string.digits
        salt = "".join(choice(allchar)
                       for x in range(randint(min_char, max_char))).encode()

        # Set the salt in the same instance if required
        if set_:
            self.set_salt(salt)

        return salt

    def set_salt(self, salt=None):
        """
            Add a salt to the secret key for this specific encryption or decryption
        """

        if salt:
            self.salted_key = salt + self.key
        else:
            self.salted_key = None

    def encrypt(self, secret):
        """
            Encrypt a secret
        """

        # generate IV
        IV = CryptoRandom.new().read(AES.block_size)

        # Retrieve AES instance
        aes = self.get_aes(IV)

        # calculate needed padding
        padding = AES.block_size - len(secret) % AES.block_size

        # Python 2.x: secret += chr(padding) * padding
        secret += bytes([padding]) * padding

        # store the IV at the beginning and encrypt
        data = IV + aes.encrypt(secret)

        # Reset salted key
        self.set_salt()

        # Return base 64 encoded bytes
        return base64.b64encode(data)

    def decrypt(self, enc_secret):
        """
            Decrypt a secret
        """

        # Decode base 64
        enc_secret = base64.b64decode(enc_secret)

        # extract the IV from the beginning
        IV = enc_secret[:AES.block_size]

        # Retrieve AES instance
        aes = self.get_aes(IV)

        # Decrypt
        data = aes.decrypt(enc_secret[AES.block_size:])

        # pick the padding value from the end; Python 2.x: ord(data[-1])
        padding = data[-1]

        # Python 2.x: chr(padding) * padding
        if data[-padding:] != bytes([padding]) * padding:
            raise ValueError("Invalid padding...")

        # Reset salted key
        self.set_salt()

        # Remove the padding and return the bytes
        return data[:-padding]
